@@ -317,7 +317,7 @@ T-INSTALLED T-UPDATABLE T-UNINSTALLED T-PENDING T-DEPRECATED."
                     (source-id (gethash "id" source))
                     (purl (mason--parse-purl source-id))
                     (version (gethash "version" purl))
-                    (version (if (not (string-match (rx bol (literal name) (any "/@-") (group (+ any)) eol) version)) version
+                    (version (if (not (string-match (rx bol (literal name) (any "/@-") (group (+ anychar)) eol) version)) version
                                (match-string 1 version)))
                     (version (replace-regexp-in-string "^[vV]" "" version))
                     (version (replace-regexp-in-string (rx bol (or "untagged-" "0.0.0-")) "" version))
@@ -332,6 +332,8 @@ T-INSTALLED T-UPDATABLE T-UNINSTALLED T-PENDING T-DEPRECATED."
                                 (or (eq 'show t-deprecated) (null deprecation))
                                 (or (string= f-category "All") (seq-contains-p categories f-category))
                                 (or (string= f-language "All") (seq-contains-p languages f-language)))))
+                 (when (and deprecation installed (eq 'show t-installed))
+                   (setq show t))
                  (when show
                    (let ((row (vector name version description "")))
                      (puthash name row mason-manager--rows)
