@@ -3,8 +3,8 @@
 ;; Copyright (C) 2025  Dimas Firmansyah
 
 ;; Author: Dimas Firmansyah <deirn@bai.lol>
-;; Package-Version: 20251117.1738
-;; Package-Revision: 27c86dc00096
+;; Package-Version: 20260107.1801
+;; Package-Revision: 47b6e24820fb
 ;; Homepage: https://github.com/deirn/mason.el
 ;; Package-Requires: ((emacs "30.1") (s "1.13.0"))
 ;; Keywords: tools lsp installer
@@ -30,7 +30,6 @@
 
 ;;; Code:
 
-(require 'ansi-color)
 (require 'cl-lib)
 (require 'json)
 (require 'seq)
@@ -270,15 +269,24 @@ To be used with `mason--process' and `mason--process-sync'."
 
 (defun mason--update-target (then)
   "Update target and call THEN."
-  (let ((output ""))
-    (mason--process
-      (mason--emacs-cmd '(message "%S" (mason--get-target)))
-      :filter (lambda (_ o) (setq output (concat output o)))
-      :then
-      (lambda (success)
-        (if (not success) (error "Updating target failed")
-          (setq mason--target (read output))
-          (funcall then))))))
+  (let* ((reg-dir (mason--expand-child-file-name "registry" mason-dir))
+         (out-file (mason--expand-child-file-name "target" reg-dir))
+         output)
+    (when (file-exists-p out-file)
+      (setq mason--target (ignore-errors (mason--read-data out-file))))
+    (if (not (null mason--target))
+        (funcall then)
+      (mason--process
+        (mason--emacs-cmd '(message "%S" (mason--get-target)))
+        :filter (lambda (_ o) (setq output (concat output o)))
+        :then
+        (lambda (success)
+          (if (not success) (error "Updating target failed")
+            (setq mason--target (read output))
+            (make-directory reg-dir t)
+            (with-temp-file out-file
+              (insert output))
+            (funcall then)))))))
 
 (defun mason--target-match (str)
   "Return non nil when target STR matches current target."
@@ -628,11 +636,50 @@ See `mason--proc-to-sexp' for TRANSFORMER."
       (mason--warn "Missing variable `%s'" var))
     (or tree "")))
 
+
+;; Global State
+
+(defvar mason--registry 'nan)
+(defvar mason--installed 'nan)
+(defvar mason--updatable 'nan)
+(defvar mason--pending (mason--make-hash))
+
+(defun mason--assert-ensured ()
+  "Assert if `mason--registry' is available."
+  (when (or (eq mason--registry 'nan)
+            (eq mason--installed 'nan)
+            (eq mason--updatable 'nan))
+    (error "Call `mason-setup' on your init.el"))
+  (when (or (eq mason--registry 'on-process)
+            (eq mason--installed 'on-process)
+            (eq mason--updatable 'on-process))
+    (error "Mason is not ready yet")))
+
+(defun mason--ensured-p ()
+  "Returns t if Mason is already setup."
+  (ignore-errors
+    (mason--assert-ensured)
+    t))
+
 
 ;; Source Resolvers
 
 (defconst mason--source-requirements nil)
 
+(defun mason--assert-source-supported (package &optional source)
+  "Errors if PACKAGE SOURCE doesn't support current platform."
+  (let* ((source (or source (gethash "source" (gethash package mason--registry))))
+         (platforms (gethash "supported_platforms" source)))
+    (when platforms
+      (unless (seq-some (lambda (x) (mason--target-match x)) platforms)
+        (error "Package `%s' only supports platforms `%s'" package (json-serialize platforms))))))
+
+(defun mason--source-supported-p (package &optional source)
+  "Returns t if PACKAGE SOURCE supports current running platform."
+  (ignore-errors
+    (mason--assert-source-supported package source)
+    t))
+
 (cl-defmacro mason--source! (type
                              (&key (req         nil)
                                    (namespace  'none)
@@ -729,10 +776,7 @@ Inside BODY, one can reference:
            (ignore name id source spec m-qualifiers
                    id-raw id-scheme id-type id-namespace id-name id-version id-qualifiers id-subpath)
            (remhash "version_overrides" source)
-           (let ((platforms (gethash "supported_platforms" source)))
-             (when platforms
-               (unless (seq-some (lambda (x) (mason--target-match x)) platforms)
-                 (error "Package `%s' only supports platforms `%s'" name (json-serialize platforms)))))
+           (mason--assert-source-supported name source)
            ,p-namespace
            ,p-version
            ,p-subpath
@@ -907,10 +951,10 @@ Expand BUILD[env] with ID."
 
 (defconst mason--github-file-regexp
   (concat "^"
-          "\\([A-ZA-Z0-9_.-]+\\)"  ; 1. file path
+          "\\([A-Za-z0-9_.-]+\\)"  ; 1. file path
           "\\("                    ; 2. optional
           ":"
-          "\\([A-ZA-Z0-9_./-]+\\)" ; 3. extract path
+          "\\([A-Za-z0-9_./-]+\\)" ; 3. extract path
           "\\)?"
           "$"))
 
@@ -963,7 +1007,7 @@ Expand BUILD[env] with ID."
         (has-build (gethash "build" source)))
     (cond
      ((and has-download has-build)
-      (error "Source `%s' has both `asset' and `build' recipe" id-raw))
+      (error "Source `%s' has both `download' and `build' recipe" id-raw))
      (has-download
       (let* ((download (mason--source-target source "download"))
              (files (gethash "files" download))
@@ -1120,22 +1164,6 @@ WIN-EXT is the extension to add when on windows."
 
 ;; The Installer
 
-(defvar mason--registry 'nan)
-(defvar mason--installed 'nan)
-(defvar mason--updatable 'nan)
-(defvar mason--pending (mason--make-hash))
-
-(defun mason--assert-ensured ()
-  "Assert if `mason--registry' is available."
-  (when (or (eq mason--registry 'nan)
-            (eq mason--installed 'nan)
-            (eq mason--updatable 'nan))
-    (error "Call `mason-ensure' on your init.el"))
-  (when (or (eq mason--registry 'on-process)
-            (eq mason--installed 'on-process)
-            (eq mason--updatable 'on-process))
-    (error "Mason is not ready yet")))
-
 (defvar mason--package-list nil)
 (defun mason--get-package-list ()
   "Get list of mason packages."
@@ -1179,9 +1207,29 @@ WIN-EXT is the extension to add when on windows."
 
 (defun mason--update-installed ()
   "Update `mason--installed'."
-  (let ((installed-index (mason--expand-child-file-name "packages/index" mason-dir)))
+  (let ((installed-index (mason--expand-child-file-name "packages/index" mason-dir))
+        resort)
     (setq mason--installed (or (mason--read-data installed-index)
-                               (mason--make-hash)))))
+                               (mason--make-hash)))
+    (maphash (lambda (key spec)
+               (unless (gethash key mason--registry)
+                 (puthash key spec mason--registry)
+                 (puthash "registry" nil spec)
+                 (unless (gethash "deprecation" spec)
+                   (puthash "deprecation"
+                            (mason--make-hash
+                              "message" "No longer in any registry"
+                              "since" "v0")
+                            spec))
+                 (setq resort t)))
+             mason--installed)
+    (when resort
+      (let* ((keys (mason--hash-keys mason--registry))
+             (keys (sort keys #'string<))
+             (reg-sorted (mason--make-hash)))
+        (dolist (key keys)
+          (puthash key (gethash key mason--registry) reg-sorted))
+        (setq mason--registry reg-sorted)))))
 
 (defun mason--update-updatable1 (spec)
   "Add SPEC to `mason--updatable' if it is updatable."
@@ -1215,9 +1263,9 @@ WIN-EXT is the extension to add when on windows."
         mason--language-list nil)
   (mason--update-target
    (lambda ()
-     (mason--update-installed)
      (let* ((reg-dir (mason--expand-child-file-name "registry" mason-dir))
-            (reg-index (mason--expand-child-file-name "index" reg-dir)))
+            (reg-index (mason--expand-child-file-name "index" reg-dir))
+            (reg-last (mason--read-data reg-index)))
        (mason--process
          (mason--emacs-cmd
            `(let ((reg (mason--make-hash))
@@ -1245,18 +1293,34 @@ WIN-EXT is the extension to add when on windows."
                 (prin1 reg (current-buffer)))))
          :then
          (lambda (success)
-           (if (not success) (error "Error downloading registry")
-             (setq mason--registry (or (mason--read-data reg-index)
-                                       (mason--make-hash)))
-             (mason--update-updatable)
+           (cond
+            (success
              (mason--success "Mason registry updated")
-             (when (functionp callback)
-               (funcall callback)))))))))
+             (setq mason--registry (or (mason--read-data reg-index)
+                                       (mason--make-hash))))
+            (reg-last
+             (mason--warn "Failed on updating registry, fallback to the old one...")
+             (setq mason--registry reg-last)
+             (make-directory reg-dir t)
+             (with-temp-file reg-index
+               (prin1 reg-last (current-buffer))))
+            (t
+             (error "Unrecoverable error when downloading the registry")))
+           (mason--update-installed)
+           (mason--update-updatable)
+           (when (functionp callback)
+             (funcall callback))))))))
 
 ;;;###autoload
-(defun mason-ensure (&optional callback)
-  "Ensure mason is setup.
-Call CALLBACK if it succeeded."
+(cl-defun mason-ensure (&optional callback)
+  "Ensure Mason is setup.
+Call CALLBACK function after setup succeeded or if it already been setup before.
+
+See `mason-setup' for the macro version of this function."
+  (when (mason--ensured-p)
+    (when (functionp callback)
+      (funcall callback))
+    (cl-return-from mason-ensure))
   (let* ((bin-dir (mason--expand-child-file-name "bin" mason-dir))
          (reg-index (mason--expand-child-file-name "registry/index" mason-dir))
          (reg-time (file-attribute-modification-time (file-attributes reg-index)))
@@ -1270,13 +1334,21 @@ Call CALLBACK if it succeeded."
           (mason-update-registry callback)
         (mason--update-target
          (lambda ()
-           (mason--update-installed)
            (setq mason--registry (mason--read-data reg-index))
+           (mason--update-installed)
            (mason--update-updatable)
            (mason--info "Mason ready")
            (when (functionp callback)
              (funcall callback))))))))
 
+;;;###autoload
+(defmacro mason-setup (&rest body)
+  "Setup Mason, then run BODY after it succeeded.
+See `mason-ensure' for the function version of this macro."
+  (declare (indent defun))
+  (if body `(mason-ensure (lambda () ,@body))
+    `(mason-ensure)))
+
 (defvar mason--ask-package-prompt nil)
 (defvar mason--ask-package-callback nil)
 (defvar mason--ask-package-category nil)
@@ -1411,6 +1483,12 @@ Call CALLBACK with the selected package spec."
            mason--package-list mason--category-list mason--language-list)
        ,@body)))
 
+;;;###autoload
+(defun mason-installed-p (package)
+  "Checks if PACKAGE is already installed."
+  (mason--assert-ensured)
+  (when (gethash package mason--installed) t))
+
 ;;;###autoload
 (defun mason-install (package &optional force interactive callback)
   "Install a Mason PACKAGE.
@@ -1622,7 +1700,16 @@ Expand TABLE from SPEC-ID-CTX and SOURCE-ID, if necessary."
 
 (defun mason-dry-run-install-all (&optional callback)
   "Dry run install all packages.
-Call CALLBACK with success and total packages."
+Call CALLBACK with success count and total packages count."
+  (if (functionp callback)
+      (mason-dry-run-install-all2 (lambda (success total _)
+                                    (funcall callback success total)))
+    (mason-dry-run-install-all2)))
+
+(defun mason-dry-run-install-all2 (&optional callback)
+  "Dry run install all packages.
+Call CALLBACK with success count, total packages count,
+and failed packages list."
   (let* ((prev-dry-run mason-dry-run)
          (prev-mason-dir mason-dir)
          (packages (mason--get-package-list))
@@ -1652,15 +1739,16 @@ Call CALLBACK with success and total packages."
                             (run-at-time
                              0 nil (lambda ()
                                      (funcall installer s)))))
+         (setq failed (reverse failed))
          (mason--info "Installed %d/%d packages, failed packages\n%s%S"
                       success-count total-count
-                      (s-repeat 8 " ") (nreverse failed))
+                      (s-repeat 8 " ") failed)
          (delete-directory mason-dir)
          (setq mason-dry-run prev-dry-run
                mason-dir prev-mason-dir)
          (mason-ensure)
          (when (functionp callback)
-           (funcall callback success-count total-count)))))
+           (funcall callback success-count total-count failed)))))
     (funcall installer nil)
     t))
 
